@@ -14,9 +14,9 @@
 
 use jj_cli::cli_util::CliRunner;
 use jj_cli::operation_templater::OperationTemplateBuildFnTable;
-use jj_cli::operation_templater::OperationTemplateLanguage;
 use jj_cli::operation_templater::OperationTemplateLanguageExtension;
-use jj_cli::template_builder::TemplateLanguage as _;
+use jj_cli::operation_templater::OperationTemplatePropertyKind;
+use jj_cli::template_builder::CoreTemplatePropertyVar as _;
 use jj_cli::template_parser;
 use jj_cli::template_parser::TemplateParseError;
 use jj_cli::templater::TemplatePropertyExt as _;
@@ -49,15 +49,14 @@ fn num_char_in_id(operation: Operation, ch_match: char) -> i64 {
 
 impl OperationTemplateLanguageExtension for HexCounter {
     fn build_fn_table(&self) -> OperationTemplateBuildFnTable {
-        type L = OperationTemplateLanguage;
+        type P = OperationTemplatePropertyKind;
         let mut table = OperationTemplateBuildFnTable::empty();
         table.operation_methods.insert(
             "num_digits_in_id",
             |_language, _diagnostics, _build_context, property, call| {
                 call.expect_no_arguments()?;
-                Ok(L::wrap_integer(
-                    property.map(|operation| num_digits_in_id(operation.id())),
-                ))
+                let out_property = property.map(|operation| num_digits_in_id(operation.id()));
+                Ok(P::wrap_integer(out_property.into_dyn()))
             },
         );
         table.operation_methods.insert(
@@ -76,9 +75,9 @@ impl OperationTemplateLanguageExtension for HexCounter {
                         }
                     })?;
 
-                Ok(L::wrap_integer(
-                    property.map(move |operation| num_char_in_id(operation, char_arg)),
-                ))
+                let out_property =
+                    property.map(move |operation| num_char_in_id(operation, char_arg));
+                Ok(P::wrap_integer(out_property.into_dyn()))
             },
         );
 
