@@ -23,6 +23,7 @@ use clap_complete::ArgValueCandidates;
 use itertools::Itertools as _;
 use jj_lib::backend;
 use jj_lib::backend::CommitId;
+use jj_lib::config::ConfigValue;
 use jj_lib::ref_name::RefName;
 use jj_lib::repo::Repo as _;
 use jj_lib::revset::RevsetExpression;
@@ -32,7 +33,7 @@ use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
 use crate::command_error::CommandError;
 use crate::commit_templater::CommitRef;
-use crate::commit_templater::CommitTemplateLanguage;
+use crate::commit_templater::CommitTemplatePropertyKind;
 use crate::complete;
 use crate::ui::Ui;
 
@@ -120,6 +121,8 @@ pub struct BookmarkListArgs {
     /// Suffix the key with `-` to sort in descending order of the value (e.g.
     /// `--sort name-`). Note that when using multiple keys, the first key is
     /// the most significant.
+    ///
+    /// This defaults to the `ui.bookmark-list-sort-keys` setting.
     #[arg(long, value_name = "SORT_KEY", value_enum, value_delimiter = ',')]
     sort: Vec<SortKey>,
 }
@@ -181,7 +184,7 @@ pub fn cmd_bookmark_list(
                 ui,
                 &language,
                 &text,
-                CommitTemplateLanguage::wrap_commit_ref,
+                CommitTemplatePropertyKind::wrap_commit_ref,
             )?
             .labeled("bookmark_list")
     };
@@ -240,9 +243,16 @@ pub fn cmd_bookmark_list(
         }
     }
 
+    let sort_keys = if args.sort.is_empty() {
+        workspace_command
+            .settings()
+            .get_value_with("ui.bookmark-list-sort-keys", parse_sort_keys)?
+    } else {
+        args.sort.clone()
+    };
     let store = repo.store();
     let mut commits: HashMap<CommitId, Arc<backend::Commit>> = HashMap::new();
-    if args.sort.iter().any(|key| key.is_commit_dependant()) {
+    if sort_keys.iter().any(|key| key.is_commit_dependant()) {
         commits = bookmark_list_items
             .iter()
             .filter_map(|item| item.primary.target().added_ids().next())
@@ -254,7 +264,7 @@ pub fn cmd_bookmark_list(
             })
             .try_collect()?;
     }
-    sort(&mut bookmark_list_items, &args.sort, &commits);
+    sort(&mut bookmark_list_items, &sort_keys, &commits);
 
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
@@ -282,8 +292,9 @@ pub fn cmd_bookmark_list(
             Some(true) => {
                 writeln!(
                     ui.hint_default(),
-                    "Bookmarks marked as deleted will be *deleted permanently* on the remote on \
-                     the next `jj git push`. Use `jj bookmark forget` to prevent this."
+                    "Bookmarks marked as deleted can be *deleted permanently* on the remote by \
+                     running `jj git push --deleted`. Use `jj bookmark forget` if you don't want \
+                     that."
                 )?;
             }
             Some(false) => {
@@ -354,6 +365,21 @@ impl SortKey {
     }
 }
 
+fn parse_sort_keys(value: ConfigValue) -> Result<Vec<SortKey>, String> {
+    if let Some(array) = value.as_array() {
+        array
+            .iter()
+            .map(|item| {
+                item.as_str()
+                    .ok_or("Expected sort key as a string".to_owned())
+                    .and_then(|key| SortKey::from_str(key, false))
+            })
+            .try_collect()
+    } else {
+        Err("Expected an array of sort keys as strings".to_owned())
+    }
+}
+
 fn sort(
     bookmark_items: &mut [RefListItem],
     sort_keys: &[SortKey],
