@@ -22,7 +22,7 @@ use crate::cli_util::CommandHelper;
 use crate::cli_util::RemoteBookmarkNamePattern;
 use crate::command_error::CommandError;
 use crate::commit_templater::CommitRef;
-use crate::commit_templater::CommitTemplateLanguage;
+use crate::commit_templater::CommitTemplatePropertyKind;
 use crate::complete;
 use crate::ui::Ui;
 
@@ -97,7 +97,7 @@ pub fn cmd_bookmark_track(
                     ui,
                     &language,
                     &text,
-                    CommitTemplateLanguage::wrap_commit_ref,
+                    CommitTemplatePropertyKind::wrap_commit_ref,
                 )?
                 .labeled("bookmark_list")
         };
