@@ -15,8 +15,9 @@
 use std::collections::HashMap;
 use std::io;
 use std::io::Read as _;
+use std::iter;
 
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
 use jj_lib::backend::Signature;
 use jj_lib::commit::CommitIteratorExt as _;
@@ -28,10 +29,12 @@ use crate::cli_util::RevisionArg;
 use crate::command_error::user_error;
 use crate::command_error::CommandError;
 use crate::complete;
+use crate::description_util::add_trailers_with_template;
 use crate::description_util::description_template;
 use crate::description_util::edit_description;
 use crate::description_util::edit_multiple_descriptions;
 use crate::description_util::join_message_paragraphs;
+use crate::description_util::parse_trailers_template;
 use crate::description_util::ParsedBulkEditMessage;
 use crate::text_util::parse_author;
 use crate::ui::Ui;
@@ -46,14 +49,14 @@ pub(crate) struct DescribeArgs {
     /// The revision(s) whose description to edit (default: @)
     #[arg(
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable)
     )]
     revisions_pos: Vec<RevisionArg>,
     #[arg(
         short = 'r',
         hide = true,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable)
     )]
     revisions_opt: Vec<RevisionArg>,
     /// The change description to use (don't open editor)
@@ -154,50 +157,58 @@ pub(crate) fn cmd_describe(
     assert!(!(args.edit && args.no_edit));
     let use_editor = args.edit || (shared_description.is_none() && !args.no_edit);
 
-    let commit_descriptions: Vec<(_, _)> = if !use_editor {
-        commits
-            .iter()
-            .map(|commit| {
-                let new_description = shared_description
-                    .as_deref()
-                    .unwrap_or_else(|| commit.description());
-                (commit, new_description.to_owned())
-            })
-            .collect()
-    } else {
-        let temp_commits: Vec<(_, _)> = commits
-            .iter()
+    let mut commit_builders = commits
+        .iter()
+        .map(|commit| {
+            let mut commit_builder = tx.repo_mut().rewrite_commit(commit).detach();
+            if let Some(description) = &shared_description {
+                commit_builder.set_description(description);
+            }
+            if args.reset_author {
+                let new_author = commit_builder.committer().clone();
+                commit_builder.set_author(new_author);
+            }
+            if let Some((name, email)) = args.author.clone() {
+                let new_author = Signature {
+                    name,
+                    email,
+                    timestamp: commit_builder.author().timestamp,
+                };
+                commit_builder.set_author(new_author);
+            }
+            commit_builder
+        })
+        .collect_vec();
+
+    if let Some(trailer_template) = parse_trailers_template(ui, &tx)? {
+        for commit_builder in &mut commit_builders {
+            // The first trailer would become the first line of the description.
+            // Also, a commit with no description is treated in a special way in jujutsu: it
+            // can be discarded as soon as it's no longer the working copy. Adding a
+            // trailer to an empty description would break that logic.
+            if use_editor || !commit_builder.description().is_empty() {
+                let temp_commit = commit_builder.write_hidden()?;
+                let new_description = add_trailers_with_template(&trailer_template, &temp_commit)?;
+                commit_builder.set_description(new_description);
+            }
+        }
+    }
+
+    if use_editor {
+        let temp_commits: Vec<_> = iter::zip(&commits, &commit_builders)
             // Edit descriptions in topological order
             .rev()
-            .map(|commit| -> Result<_, CommandError> {
-                let mut commit_builder = tx.repo_mut().rewrite_commit(commit).detach();
-                if let Some(description) = &shared_description {
-                    commit_builder.set_description(description);
-                } else if commit_builder.description().is_empty() {
-                    commit_builder
-                        .set_description(tx.settings().get_string("ui.default-description")?);
-                }
-                if args.reset_author {
-                    let new_author = commit_builder.committer().clone();
-                    commit_builder.set_author(new_author);
-                }
-                if let Some((name, email)) = args.author.clone() {
-                    let new_author = Signature {
-                        name,
-                        email,
-                        timestamp: commit_builder.author().timestamp,
-                    };
-                    commit_builder.set_author(new_author);
-                }
-                let temp_commit = commit_builder.write_hidden()?;
-                Ok((commit.id(), temp_commit))
+            .map(|(commit, commit_builder)| {
+                commit_builder
+                    .write_hidden()
+                    .map(|temp_commit| (commit.id(), temp_commit))
             })
             .try_collect()?;
 
         if let [(_, temp_commit)] = &*temp_commits {
             let template = description_template(ui, &tx, "", temp_commit)?;
             let description = edit_description(&text_editor, &template)?;
-            vec![(&commits[0], description)]
+            commit_builders[0].set_description(description);
         } else {
             let ParsedBulkEditMessage {
                 descriptions,
@@ -226,30 +237,25 @@ pub(crate) fn cmd_describe(
                 )));
             }
 
-            let commit_descriptions = commits
-                .iter()
-                .map(|commit| {
-                    let description = descriptions.get(commit.id()).unwrap().to_owned();
-                    (commit, description)
-                })
-                .collect();
-
-            commit_descriptions
+            for (commit, commit_builder) in iter::zip(&commits, &mut commit_builders) {
+                let description = descriptions.get(commit.id()).unwrap();
+                commit_builder.set_description(description);
+            }
         }
     };
 
     // Filter out unchanged commits to avoid rebasing descendants in
     // `transform_descendants` below unnecessarily.
-    let commit_descriptions: HashMap<_, _> = commit_descriptions
-        .into_iter()
-        .filter(|(commit, new_description)| {
-            new_description != commit.description()
+    let commit_builders: HashMap<_, _> = iter::zip(&commits, commit_builders)
+        .filter(|(old_commit, commit_builder)| {
+            old_commit.description() != commit_builder.description()
                 || args.reset_author
-                || args.author.as_ref().is_some_and(|(name, email)| {
-                    name != &commit.author().name || email != &commit.author().email
-                })
+                // Ignore author timestamp which could be updated if the old
+                // commit was discardable.
+                || old_commit.author().name != commit_builder.author().name
+                || old_commit.author().email != commit_builder.author().email
         })
-        .map(|(commit, new_description)| (commit.id(), new_description))
+        .map(|(old_commit, commit_builder)| (old_commit.id(), commit_builder))
         .collect();
 
     let mut num_described = 0;
@@ -260,32 +266,22 @@ pub(crate) fn cmd_describe(
     // rewriting the same commit multiple times, and adding additional entries
     // in the predecessor chain.
     tx.repo_mut().transform_descendants(
-        commit_descriptions
-            .keys()
-            .map(|&id| id.clone())
-            .collect_vec(),
+        commit_builders.keys().map(|&id| id.clone()).collect(),
         |rewriter| {
             let old_commit_id = rewriter.old_commit().id().clone();
-            let mut commit_builder = rewriter.reparent();
-            if let Some(description) = commit_descriptions.get(&old_commit_id) {
-                commit_builder = commit_builder.set_description(description);
-                if args.reset_author {
-                    let new_author = commit_builder.committer().clone();
-                    commit_builder = commit_builder.set_author(new_author);
-                }
-                if let Some((name, email)) = args.author.clone() {
-                    let new_author = Signature {
-                        name,
-                        email,
-                        timestamp: commit_builder.author().timestamp,
-                    };
-                    commit_builder = commit_builder.set_author(new_author);
-                }
+            let commit_builder = rewriter.reparent();
+            if let Some(temp_builder) = commit_builders.get(&old_commit_id) {
+                commit_builder
+                    .set_description(temp_builder.description())
+                    .set_author(temp_builder.author().clone())
+                    // Copy back committer for consistency with author timestamp
+                    .set_committer(temp_builder.committer().clone())
+                    .write()?;
                 num_described += 1;
             } else {
+                commit_builder.write()?;
                 num_reparented += 1;
             }
-            commit_builder.write()?;
             Ok(())
         },
     )?;
