@@ -12,13 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
 use crate::command_error::CommandError;
-use crate::commit_templater::CommitTemplateLanguage;
+use crate::commit_templater::CommitTemplatePropertyKind;
 use crate::commit_templater::TreeEntry;
 use crate::complete;
 use crate::ui::Ui;
@@ -31,7 +31,7 @@ pub(crate) struct FileListArgs {
         long, short,
         default_value = "@",
         value_name = "REVSET",
-        add = ArgValueCandidates::new(complete::all_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     revision: RevisionArg,
 
@@ -77,7 +77,7 @@ pub(crate) fn cmd_file_list(
                 ui,
                 &language,
                 &text,
-                CommitTemplateLanguage::wrap_tree_entry,
+                CommitTemplatePropertyKind::wrap_tree_entry,
             )?
             .labeled("file_list")
     };
