@@ -12,33 +12,35 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::io::Write;
+use std::io;
+use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
+use std::str;
 use std::sync::Arc;
 
+use itertools::Itertools as _;
 use jj_lib::file_util;
 use jj_lib::git;
 use jj_lib::git::parse_git_ref;
-use jj_lib::git::RefName;
-use jj_lib::refs::RemoteRefSymbol;
+use jj_lib::git::GitRefKind;
 use jj_lib::repo::ReadonlyRepo;
-use jj_lib::repo::Repo;
+use jj_lib::repo::Repo as _;
+use jj_lib::view::View;
 use jj_lib::workspace::Workspace;
 
 use super::write_repository_level_trunk_alias;
-use crate::cli_util::print_trackable_remote_bookmarks;
 use crate::cli_util::start_repo_transaction;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::WorkspaceCommandHelper;
 use crate::command_error::cli_error;
+use crate::command_error::internal_error;
 use crate::command_error::user_error_with_hint;
 use crate::command_error::user_error_with_message;
 use crate::command_error::CommandError;
 use crate::commands::git::maybe_add_gitignore;
-use crate::git_util::get_git_repo;
 use crate::git_util::is_colocated_git_workspace;
-use crate::git_util::print_failed_git_export;
+use crate::git_util::print_git_export_stats;
 use crate::git_util::print_git_import_stats;
 use crate::ui::Ui;
 
@@ -208,21 +210,16 @@ fn init_git_refs(
     let mut tx = start_repo_transaction(&repo, string_args);
     // There should be no old refs to abandon, but enforce it.
     git_settings.abandon_unreachable_commits = false;
-    let stats = git::import_some_refs(
-        tx.repo_mut(),
-        &git_settings,
-        // Initial import shouldn't fail because of reserved remote name.
-        |ref_name| !git::is_reserved_git_remote_ref(ref_name),
-    )?;
+    let stats = git::import_refs(tx.repo_mut(), &git_settings)?;
+    print_git_import_stats(ui, tx.repo(), &stats, false)?;
     if !tx.repo().has_changes() {
         return Ok(repo);
     }
-    print_git_import_stats(ui, tx.repo(), &stats, false)?;
     if colocated {
         // If git.auto-local-bookmark = true, local bookmarks could be created for
         // the imported remote branches.
-        let failed_refs = git::export_refs(tx.repo_mut())?;
-        print_failed_git_export(ui, &failed_refs)?;
+        let stats = git::export_refs(tx.repo_mut())?;
+        print_git_export_stats(ui, &stats)?;
     }
     let repo = tx.commit("import git refs")?;
     writeln!(
@@ -237,15 +234,18 @@ pub fn maybe_set_repository_level_trunk_alias(
     ui: &Ui,
     workspace_command: &WorkspaceCommandHelper,
 ) -> Result<(), CommandError> {
-    let git_repo = get_git_repo(workspace_command.repo().store())?;
-    if let Ok(reference) = git_repo.find_reference("refs/remotes/origin/HEAD") {
-        if let Some(reference_name) = reference.symbolic_target() {
-            if let Some(RefName::RemoteBranch(symbol)) = parse_git_ref(reference_name) {
+    let git_repo = git::get_git_repo(workspace_command.repo().store())?;
+    if let Some(reference) = git_repo
+        .try_find_reference("refs/remotes/origin/HEAD")
+        .map_err(internal_error)?
+    {
+        if let Some(reference_name) = reference.target().try_name() {
+            if let Some((GitRefKind::Bookmark, symbol)) = str::from_utf8(reference_name.as_bstr())
+                .ok()
+                .and_then(|name| parse_git_ref(name.as_ref()))
+            {
                 // TODO: Can we assume the symbolic target points to the same remote?
-                let symbol = RemoteRefSymbol {
-                    name: &symbol.name,
-                    remote: "origin",
-                };
+                let symbol = symbol.name.to_remote_symbol("origin".as_ref());
                 write_repository_level_trunk_alias(ui, workspace_command.repo_path(), symbol)?;
             }
         };
@@ -253,3 +253,37 @@ pub fn maybe_set_repository_level_trunk_alias(
 
     Ok(())
 }
+
+fn print_trackable_remote_bookmarks(ui: &Ui, view: &View) -> io::Result<()> {
+    let remote_bookmark_symbols = view
+        .bookmarks()
+        .filter(|(_, bookmark_target)| bookmark_target.local_target.is_present())
+        .flat_map(|(name, bookmark_target)| {
+            bookmark_target
+                .remote_refs
+                .into_iter()
+                .filter(|&(_, remote_ref)| !remote_ref.is_tracked())
+                .map(move |(remote, _)| name.to_remote_symbol(remote))
+        })
+        .collect_vec();
+    if remote_bookmark_symbols.is_empty() {
+        return Ok(());
+    }
+
+    if let Some(mut formatter) = ui.status_formatter() {
+        writeln!(
+            formatter.labeled("hint").with_heading("Hint: "),
+            "The following remote bookmarks aren't associated with the existing local bookmarks:"
+        )?;
+        for symbol in &remote_bookmark_symbols {
+            write!(formatter, "  ")?;
+            writeln!(formatter.labeled("bookmark"), "{symbol}")?;
+        }
+        writeln!(
+            formatter.labeled("hint").with_heading("Hint: "),
+            "Run `jj bookmark track {syms}` to keep local bookmarks updated on future pulls.",
+            syms = remote_bookmark_symbols.iter().join(" "),
+        )?;
+    }
+    Ok(())
+}
