@@ -35,7 +35,7 @@ use crate::cli_util::CommandHelper;
 use crate::cli_util::LogContentFormat;
 use crate::cli_util::RevisionArg;
 use crate::command_error::CommandError;
-use crate::commit_templater::CommitTemplateLanguage;
+use crate::commit_templater::CommitTemplatePropertyKind;
 use crate::complete;
 use crate::diff_util::DiffFormatArgs;
 use crate::graphlog::get_graphlog;
@@ -74,7 +74,7 @@ pub(crate) struct LogArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::all_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     revisions: Vec<RevisionArg>,
     /// Show revisions modifying the given paths
@@ -176,7 +176,7 @@ pub(crate) fn cmd_log(
                 ui,
                 &language,
                 &template_string,
-                CommitTemplateLanguage::wrap_commit,
+                CommitTemplatePropertyKind::wrap_commit,
             )?
             .labeled("log");
         node_template = workspace_command
@@ -184,7 +184,7 @@ pub(crate) fn cmd_log(
                 ui,
                 &language,
                 &get_node_template(graph_style, settings)?,
-                CommitTemplateLanguage::wrap_commit_opt,
+                CommitTemplatePropertyKind::wrap_commit_opt,
             )?
             .labeled("node");
     }
