@@ -42,6 +42,7 @@ mod prev;
 mod rebase;
 mod resolve;
 mod restore;
+mod revert;
 mod root;
 mod run;
 mod show;
@@ -53,7 +54,6 @@ mod squash;
 mod status;
 mod tag;
 mod unsign;
-mod unsquash;
 mod util;
 mod version;
 mod workspace;
@@ -62,9 +62,9 @@ use std::fmt::Debug;
 
 use clap::builder::styling::AnsiColor;
 use clap::builder::Styles;
-use clap::CommandFactory;
-use clap::FromArgMatches;
-use clap::Subcommand;
+use clap::CommandFactory as _;
+use clap::FromArgMatches as _;
+use clap::Subcommand as _;
 use clap_complete::engine::SubcommandCandidates;
 use tracing::instrument;
 
@@ -88,6 +88,7 @@ const STYLES: Styles = Styles::styled()
 enum Command {
     Abandon(abandon::AbandonArgs),
     Absorb(absorb::AbsorbArgs),
+    // TODO: Remove in jj 0.34+
     Backout(backout::BackoutArgs),
     #[cfg(feature = "bench")]
     #[command(subcommand)]
@@ -128,6 +129,7 @@ enum Command {
     Rebase(rebase::RebaseArgs),
     Resolve(resolve::ResolveArgs),
     Restore(restore::RestoreArgs),
+    Revert(revert::RevertArgs),
     Root(root::RootArgs),
     #[command(hide = true)]
     // TODO: Flesh out.
@@ -147,12 +149,6 @@ enum Command {
     /// Undo an operation (shortcut for `jj op undo`)
     Undo(operation::undo::OperationUndoArgs),
     Unsign(unsign::UnsignArgs),
-    // TODO: Delete `unsquash` in jj 0.28+
-    #[command(hide = true)]
-    Unsquash(unsquash::UnsquashArgs),
-    // TODO: Delete `untrack` in jj 0.27+
-    #[command(hide = true)]
-    Untrack(file::untrack::FileUntrackArgs),
     Version(version::VersionArgs),
     #[command(subcommand)]
     Workspace(workspace::WorkspaceCommand),
@@ -200,6 +196,7 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Rebase(args) => rebase::cmd_rebase(ui, command_helper, args),
         Command::Resolve(args) => resolve::cmd_resolve(ui, command_helper, args),
         Command::Restore(args) => restore::cmd_restore(ui, command_helper, args),
+        Command::Revert(args) => revert::cmd_revert(ui, command_helper, args),
         Command::Root(args) => root::cmd_root(ui, command_helper, args),
         Command::Run(args) => run::cmd_run(ui, command_helper, args),
         Command::SimplifyParents(args) => {
@@ -214,11 +211,6 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Tag(args) => tag::cmd_tag(ui, command_helper, args),
         Command::Undo(args) => operation::undo::cmd_op_undo(ui, command_helper, args),
         Command::Unsign(args) => unsign::cmd_unsign(ui, command_helper, args),
-        Command::Unsquash(args) => unsquash::cmd_unsquash(ui, command_helper, args),
-        Command::Untrack(args) => {
-            let cmd = renamed_cmd("untrack", "file untrack", file::untrack::cmd_file_untrack);
-            cmd(ui, command_helper, args)
-        }
         Command::Util(args) => util::cmd_util(ui, command_helper, args),
         Command::Version(args) => version::cmd_version(ui, command_helper, args),
         Command::Workspace(args) => workspace::cmd_workspace(ui, command_helper, args),
