@@ -19,7 +19,7 @@ use super::diff::show_op_diff;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::LogContentFormat;
 use crate::command_error::CommandError;
-use crate::commit_templater::CommitTemplateLanguage;
+use crate::commit_templater::CommitTemplatePropertyKind;
 use crate::complete;
 use crate::diff_util::diff_formats_for_log;
 use crate::diff_util::DiffFormatArgs;
@@ -66,7 +66,12 @@ pub fn cmd_op_show(
     let commit_summary_template = {
         let language = workspace_env.commit_template_language(repo.as_ref(), &id_prefix_context);
         let text = settings.get_string("templates.commit_summary")?;
-        workspace_env.parse_template(ui, &language, &text, CommitTemplateLanguage::wrap_commit)?
+        workspace_env.parse_template(
+            ui,
+            &language,
+            &text,
+            CommitTemplatePropertyKind::wrap_commit,
+        )?
     };
 
     let graph_style = GraphStyle::from_settings(settings)?;
