@@ -16,7 +16,7 @@ use std::io::Write as _;
 use std::sync::Arc;
 
 use clap::ArgGroup;
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
 use jj_lib::backend::CommitId;
 use jj_lib::commit::Commit;
@@ -278,7 +278,7 @@ pub(crate) struct RebaseArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     branch: Vec<RevisionArg>,
 
@@ -294,7 +294,7 @@ pub(crate) struct RebaseArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     source: Vec<RevisionArg>,
     /// Rebase the given revisions, rebasing descendants onto this revision's
@@ -308,7 +308,7 @@ pub(crate) struct RebaseArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     revisions: Vec<RevisionArg>,
 
@@ -336,7 +336,7 @@ pub struct RebaseDestinationArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::all_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     destination: Option<Vec<RevisionArg>>,
     /// The revision(s) to insert after (can be repeated to create a merge
@@ -347,7 +347,7 @@ pub struct RebaseDestinationArgs {
         visible_alias = "after",
         conflicts_with = "destination",
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::all_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     insert_after: Option<Vec<RevisionArg>>,
     /// The revision(s) to insert before (can be repeated to create a merge
@@ -358,7 +358,7 @@ pub struct RebaseDestinationArgs {
         visible_alias = "before",
         conflicts_with = "destination",
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     insert_before: Option<Vec<RevisionArg>>,
 }
@@ -386,41 +386,46 @@ pub(crate) fn cmd_rebase(
         simplify_ancestor_merge: false,
     };
     let mut workspace_command = command.workspace_helper(ui)?;
-    if !args.revisions.is_empty() {
-        rebase_revisions(
-            ui,
-            &mut workspace_command,
-            &args.revisions,
-            &args.destination,
-            &rebase_options,
-        )?;
+    let plan = if !args.revisions.is_empty() {
+        plan_rebase_revisions(ui, &workspace_command, &args.revisions, &args.destination)?
     } else if !args.source.is_empty() {
-        rebase_source(
-            ui,
-            &mut workspace_command,
-            &args.source,
-            &args.destination,
-            &rebase_options,
-        )?;
+        plan_rebase_source(ui, &workspace_command, &args.source, &args.destination)?
     } else {
-        rebase_branch(
-            ui,
-            &mut workspace_command,
-            &args.branch,
-            &args.destination,
-            &rebase_options,
-        )?;
-    }
+        plan_rebase_branch(ui, &workspace_command, &args.branch, &args.destination)?
+    };
+
+    let mut tx = workspace_command.start_transaction();
+    let new_children: Vec<_> = plan
+        .new_child_ids
+        .iter()
+        .map(|commit_id| tx.repo().store().get_commit(commit_id))
+        .try_collect()?;
+    let stats = move_commits(
+        tx.repo_mut(),
+        &plan.new_parent_ids,
+        &new_children,
+        &plan.target,
+        &rebase_options,
+    )?;
+    print_move_commits_stats(ui, &stats)?;
+    tx.finish(ui, tx_description(&plan.target))?;
+
     Ok(())
 }
 
-fn rebase_revisions(
-    ui: &mut Ui,
-    workspace_command: &mut WorkspaceCommandHelper,
+#[derive(Clone, Debug)]
+struct RebasePlan {
+    new_parent_ids: Vec<CommitId>,
+    new_child_ids: Vec<CommitId>,
+    target: MoveCommitsTarget,
+}
+
+fn plan_rebase_revisions(
+    ui: &Ui,
+    workspace_command: &WorkspaceCommandHelper,
     revisions: &[RevisionArg],
     rebase_destination: &RebaseDestinationArgs,
-    rebase_options: &RebaseOptions,
-) -> Result<(), CommandError> {
+) -> Result<RebasePlan, CommandError> {
     let target_commits: Vec<_> = workspace_command
         .parse_union_revsets(ui, revisions)?
         .evaluate_to_commits()?
@@ -445,23 +450,19 @@ fn rebase_revisions(
             }
         }
     }
-    rebase_revisions_transaction(
-        ui,
-        workspace_command,
-        &new_parent_ids,
-        &new_child_ids,
-        target_commits,
-        rebase_options,
-    )
+    Ok(RebasePlan {
+        new_parent_ids,
+        new_child_ids,
+        target: MoveCommitsTarget::Commits(target_commits),
+    })
 }
 
-fn rebase_source(
-    ui: &mut Ui,
-    workspace_command: &mut WorkspaceCommandHelper,
+fn plan_rebase_source(
+    ui: &Ui,
+    workspace_command: &WorkspaceCommandHelper,
     source: &[RevisionArg],
     rebase_destination: &RebaseDestinationArgs,
-    rebase_options: &RebaseOptions,
-) -> Result<(), CommandError> {
+) -> Result<RebasePlan, CommandError> {
     let source_commits: Vec<_> = workspace_command
         .resolve_some_revsets_default_single(ui, source)?
         .iter()
@@ -483,23 +484,19 @@ fn rebase_source(
         }
     }
 
-    rebase_descendants_transaction(
-        ui,
-        workspace_command,
-        &new_parent_ids,
-        &new_child_ids,
-        source_commits,
-        rebase_options,
-    )
+    Ok(RebasePlan {
+        new_parent_ids,
+        new_child_ids,
+        target: MoveCommitsTarget::Roots(source_commits),
+    })
 }
 
-fn rebase_branch(
-    ui: &mut Ui,
-    workspace_command: &mut WorkspaceCommandHelper,
+fn plan_rebase_branch(
+    ui: &Ui,
+    workspace_command: &WorkspaceCommandHelper,
     branch: &[RevisionArg],
     rebase_destination: &RebaseDestinationArgs,
-    rebase_options: &RebaseOptions,
-) -> Result<(), CommandError> {
+) -> Result<RebasePlan, CommandError> {
     let branch_commit_ids: Vec<_> = if branch.is_empty() {
         vec![workspace_command
             .resolve_single_rev(ui, &RevisionArg::AT)?
@@ -536,95 +533,11 @@ fn rebase_branch(
         }
     }
 
-    rebase_descendants_transaction(
-        ui,
-        workspace_command,
-        &new_parent_ids,
-        &new_child_ids,
-        root_commits,
-        rebase_options,
-    )
-}
-
-fn rebase_descendants_transaction(
-    ui: &mut Ui,
-    workspace_command: &mut WorkspaceCommandHelper,
-    new_parent_ids: &[CommitId],
-    new_child_ids: &[CommitId],
-    target_roots: Vec<Commit>,
-    rebase_options: &RebaseOptions,
-) -> Result<(), CommandError> {
-    if target_roots.is_empty() {
-        writeln!(ui.status(), "Nothing changed.")?;
-        return Ok(());
-    }
-
-    let mut tx = workspace_command.start_transaction();
-    let tx_description = if target_roots.len() == 1 {
-        format!(
-            "rebase commit {} and descendants",
-            target_roots.first().unwrap().id().hex()
-        )
-    } else {
-        format!(
-            "rebase {} commits and their descendants",
-            target_roots.len()
-        )
-    };
-
-    let new_children: Vec<_> = new_child_ids
-        .iter()
-        .map(|commit_id| tx.repo().store().get_commit(commit_id))
-        .try_collect()?;
-    let stats = move_commits(
-        tx.repo_mut(),
+    Ok(RebasePlan {
         new_parent_ids,
-        &new_children,
-        &MoveCommitsTarget::Roots(target_roots),
-        rebase_options,
-    )?;
-    print_move_commits_stats(ui, &stats)?;
-    tx.finish(ui, tx_description)
-}
-
-/// Creates a transaction for rebasing revisions.
-fn rebase_revisions_transaction(
-    ui: &mut Ui,
-    workspace_command: &mut WorkspaceCommandHelper,
-    new_parent_ids: &[CommitId],
-    new_child_ids: &[CommitId],
-    target_commits: Vec<Commit>,
-    rebase_options: &RebaseOptions,
-) -> Result<(), CommandError> {
-    if target_commits.is_empty() {
-        writeln!(ui.status(), "Nothing changed.")?;
-        return Ok(());
-    }
-
-    let mut tx = workspace_command.start_transaction();
-    let tx_description = if target_commits.len() == 1 {
-        format!("rebase commit {}", target_commits[0].id().hex())
-    } else {
-        format!(
-            "rebase commit {} and {} more",
-            target_commits[0].id().hex(),
-            target_commits.len() - 1
-        )
-    };
-
-    let new_children: Vec<_> = new_child_ids
-        .iter()
-        .map(|commit_id| tx.repo().store().get_commit(commit_id))
-        .try_collect()?;
-    let stats = move_commits(
-        tx.repo_mut(),
-        new_parent_ids,
-        &new_children,
-        &MoveCommitsTarget::Commits(target_commits),
-        rebase_options,
-    )?;
-    print_move_commits_stats(ui, &stats)?;
-    tx.finish(ui, tx_description)
+        new_child_ids,
+        target: MoveCommitsTarget::Roots(root_commits),
+    })
 }
 
 fn check_rebase_destinations(
@@ -644,6 +557,24 @@ fn check_rebase_destinations(
     Ok(())
 }
 
+fn tx_description(target: &MoveCommitsTarget) -> String {
+    match &target {
+        MoveCommitsTarget::Commits(commits) => match &commits[..] {
+            [] => format!("rebase {} commits", commits.len()),
+            [commit] => format!("rebase commit {}", commit.id().hex()),
+            [first, others @ ..] => format!(
+                "rebase commit {} and {} more",
+                first.id().hex(),
+                others.len()
+            ),
+        },
+        MoveCommitsTarget::Roots(commits) => match &commits[..] {
+            [commit] => format!("rebase commit {} and descendants", commit.id().hex()),
+            _ => format!("rebase {} commits and their descendants", commits.len()),
+        },
+    }
+}
+
 /// Print details about the provided [`MoveCommitsStats`].
 fn print_move_commits_stats(ui: &Ui, stats: &MoveCommitsStats) -> std::io::Result<()> {
     let Some(mut formatter) = ui.status_formatter() else {
