@@ -30,6 +30,8 @@ use crossterm::terminal::Clear;
 use crossterm::terminal::ClearType;
 use indoc::writedoc;
 use itertools::Itertools as _;
+#[cfg(feature = "git2")]
+use jj_lib::config::ConfigGetResultExt as _;
 use jj_lib::fmt_util::binary_prefix;
 use jj_lib::git;
 use jj_lib::git::FailedRefExportReason;
@@ -41,6 +43,8 @@ use jj_lib::op_store::RemoteRef;
 use jj_lib::ref_name::RemoteRefSymbol;
 use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo::Repo;
+#[cfg(feature = "git2")]
+use jj_lib::settings::UserSettings;
 use jj_lib::workspace::Workspace;
 use unicode_width::UnicodeWidthStr as _;
 
@@ -149,8 +153,8 @@ fn pinentry_get_pw(url: &str) -> Option<String> {
 #[tracing::instrument]
 fn get_ssh_keys(_username: &str) -> Vec<PathBuf> {
     let mut paths = vec![];
-    if let Some(home_dir) = dirs::home_dir() {
-        let ssh_dir = Path::new(&home_dir).join(".ssh");
+    if let Ok(home_dir) = etcetera::home_dir() {
+        let ssh_dir = home_dir.join(".ssh");
         for filename in ["id_ed25519_sk", "id_ed25519", "id_rsa"] {
             let key_path = ssh_dir.join(filename);
             if key_path.is_file() {
@@ -612,6 +616,26 @@ export or their "parent" bookmarks."#,
     Ok(())
 }
 
+#[cfg(feature = "git2")]
+pub fn print_git2_deprecation_warning(
+    ui: &Ui,
+    settings: &UserSettings,
+) -> Result<(), CommandError> {
+    if !settings.git_settings()?.subprocess
+        && !settings
+            .get("debug.suppress-git2-deprecation-warning")
+            .optional()?
+            .unwrap_or(false)
+    {
+        writeln!(
+            ui.warning_default(),
+            "`git.subprocess = false` will be removed in 0.30; please report any issues you have \
+             with the default.",
+        )?;
+    }
+    Ok(())
+}
+
 #[cfg(test)]
 mod tests {
     use std::path::MAIN_SEPARATOR;
