@@ -18,16 +18,16 @@ mod external;
 
 use std::sync::Arc;
 
-use bstr::BString;
 use itertools::Itertools as _;
 use jj_lib::backend::BackendError;
-use jj_lib::backend::FileId;
 use jj_lib::backend::MergedTreeId;
+use jj_lib::backend::TreeValue;
 use jj_lib::config::ConfigGetError;
 use jj_lib::config::ConfigGetResultExt as _;
 use jj_lib::config::ConfigNamePathBuf;
-use jj_lib::conflicts::extract_as_single_hunk;
+use jj_lib::conflicts::try_materialize_file_conflict_value;
 use jj_lib::conflicts::ConflictMarkerStyle;
+use jj_lib::conflicts::MaterializedFileConflictValue;
 use jj_lib::gitignore::GitIgnoreFile;
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::Merge;
@@ -96,12 +96,15 @@ pub enum ConflictResolveError {
     #[error("Couldn't find any conflicts at {0:?} in this revision")]
     NotAConflict(RepoPathBuf),
     #[error(
-        "Only conflicts that involve normal files (not symlinks, not executable, etc.) are \
-         supported. Conflict summary for {0:?}:\n{1}"
+        "Only conflicts that involve normal files (not symlinks, etc.) are supported. Conflict \
+         summary for {path:?}:\n{summary}",
+        summary = summary.trim_end()
     )]
-    NotNormalFiles(RepoPathBuf, String),
+    NotNormalFiles { path: RepoPathBuf, summary: String },
     #[error("The conflict at {path:?} has {sides} sides. At most 2 sides are supported.")]
     ConflictTooComplicated { path: RepoPathBuf, sides: usize },
+    #[error("{path:?} has conflicts in executable bit\n{summary}", summary = summary.trim_end())]
+    ExecutableConflict { path: RepoPathBuf, summary: String },
     #[error(
         "The output file is either unchanged or empty after the editor quit (run with --debug to \
          see the exact invocation)."
@@ -315,9 +318,7 @@ impl DiffEditor {
 struct MergeToolFile {
     repo_path: RepoPathBuf,
     conflict: MergedTreeValue,
-    file_merge: Merge<Option<FileId>>,
-    simplified_file_merge: Merge<Option<FileId>>,
-    simplified_file_content: Merge<BString>,
+    file: MaterializedFileConflictValue,
 }
 
 impl MergeToolFile {
@@ -330,26 +331,29 @@ impl MergeToolFile {
             Ok(Some(_)) => return Err(ConflictResolveError::NotAConflict(repo_path.to_owned())),
             Ok(None) => return Err(ConflictResolveError::PathNotFound(repo_path.to_owned())),
         };
-        let file_merge = conflict.to_file_merge().ok_or_else(|| {
-            let summary = conflict.describe();
-            ConflictResolveError::NotNormalFiles(repo_path.to_owned(), summary)
-        })?;
-        let simplified_file_merge = file_merge.clone().simplify();
+        let file = try_materialize_file_conflict_value(tree.store(), repo_path, &conflict)
+            .block_on()?
+            .ok_or_else(|| ConflictResolveError::NotNormalFiles {
+                path: repo_path.to_owned(),
+                summary: conflict.describe(),
+            })?;
         // We only support conflicts with 2 sides (3-way conflicts)
-        if simplified_file_merge.num_sides() > 2 {
+        if file.ids.num_sides() > 2 {
             return Err(ConflictResolveError::ConflictTooComplicated {
                 path: repo_path.to_owned(),
-                sides: simplified_file_merge.num_sides(),
+                sides: file.ids.num_sides(),
             });
         };
-        let simplified_file_content =
-            extract_as_single_hunk(&simplified_file_merge, tree.store(), repo_path).block_on()?;
+        if file.executable.is_none() {
+            return Err(ConflictResolveError::ExecutableConflict {
+                path: repo_path.to_owned(),
+                summary: conflict.describe(),
+            });
+        }
         Ok(MergeToolFile {
             repo_path: repo_path.to_owned(),
             conflict,
-            file_merge,
-            simplified_file_merge,
-            simplified_file_content,
+            file,
         })
     }
 }
@@ -457,20 +461,11 @@ fn pick_conflict_side(
     for merge_tool_file in merge_tool_files {
         // We use file IDs here to match the logic for the other external merge tools.
         // This ensures that the behavior is consistent.
-        let file_id = merge_tool_file
-            .simplified_file_merge
-            .get_add(add_index)
-            .unwrap();
-        // Update the file IDs only, leaving the executable flags unchanged
-        let new_tree_value = merge_tool_file
-            .conflict
-            .with_new_file_ids(&Merge::from_vec(vec![
-                file_id.clone();
-                merge_tool_file
-                    .conflict
-                    .as_slice()
-                    .len()
-            ]));
+        let file = &merge_tool_file.file;
+        let file_id = file.ids.get_add(add_index).unwrap();
+        let executable = file.executable.expect("should have been resolved");
+        let new_tree_value =
+            Merge::resolved(file_id.clone().map(|id| TreeValue::File { id, executable }));
         tree_builder.set_or_remove(merge_tool_file.repo_path.clone(), new_tree_value);
     }
     tree_builder.write_tree(tree.store())
