@@ -394,6 +394,8 @@ tuple_impls! {
     (0 T0, 1 T1, 2 T2, 3 T3)
 }
 
+pub type BoxedTemplateProperty<'a, O> = Box<dyn TemplateProperty<Output = O> + 'a>;
+
 /// `TemplateProperty` adapters that are useful when implementing methods.
 pub trait TemplatePropertyExt: TemplateProperty {
     /// Translates to a property that will apply fallible `function` to an
@@ -435,6 +437,14 @@ pub trait TemplatePropertyExt: TemplateProperty {
     {
         Box::new(FormattablePropertyTemplate::new(self))
     }
+
+    /// Converts this property into boxed trait object.
+    fn into_dyn<'a>(self) -> BoxedTemplateProperty<'a, Self::Output>
+    where
+        Self: Sized + 'a,
+    {
+        Box::new(self)
+    }
 }
 
 impl<P: TemplateProperty + ?Sized> TemplatePropertyExt for P {}
