@@ -1576,7 +1576,7 @@ fn test_bookmark_list() {
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -1592,7 +1592,7 @@ fn test_bookmark_list() {
     remote-untrack@origin: vmortlor 71a16b05 (empty) remote-untrack
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -1694,7 +1694,7 @@ fn test_bookmark_list() {
     tracking_behind_count: <Error: Not a tracked remote ref>
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 }
@@ -1771,7 +1771,7 @@ fn test_bookmark_list_filtered() {
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -1835,7 +1835,7 @@ fn test_bookmark_list_filtered() {
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
     insta::assert_snapshot!(query(&["--remote", "glob:gi?"]), @r"
@@ -1860,7 +1860,7 @@ fn test_bookmark_list_filtered() {
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -1870,7 +1870,7 @@ fn test_bookmark_list_filtered() {
       @origin: yxusvupt dad5f298 (empty) remote-delete
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
     insta::assert_snapshot!(query(&["-rbookmarks(remote-delete)"]), @"");
@@ -1890,7 +1890,7 @@ fn test_bookmark_list_filtered() {
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -2113,7 +2113,7 @@ fn test_bookmark_list_tracked() {
       @upstream: lolpmnqw 32fa6da0 (empty) upstream-sync
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -2130,7 +2130,7 @@ fn test_bookmark_list_tracked() {
       @upstream: lolpmnqw 32fa6da0 (empty) upstream-sync
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -2144,7 +2144,7 @@ fn test_bookmark_list_tracked() {
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
@@ -2234,10 +2234,10 @@ fn test_bookmark_list_sort_multiple_keys() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    for (bookmark, committeremail) in [("c", "bob@g.c"), ("b", "alice@g.c"), ("a", "bob@g.c")] {
+    for (bookmark, email) in [("c", "bob@g.c"), ("b", "alice@g.c"), ("a", "bob@g.c")] {
         work_dir
             .run_jj([
-                &format!("--config=user.email={committeremail}"),
+                &format!("--config=user.email={email}"),
                 "new",
                 "root()",
                 "-m",
@@ -2259,6 +2259,77 @@ fn test_bookmark_list_sort_multiple_keys() {
     ");
 }
 
+#[test]
+fn test_bookmark_list_sort_using_config() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+    for (bookmark, email) in [("c", "bob@g.c"), ("b", "alice@g.c"), ("a", "bob@g.c")] {
+        work_dir
+            .run_jj([
+                &format!("--config=user.email={email}"),
+                "new",
+                "root()",
+                "-m",
+                "fix",
+            ])
+            .success();
+        work_dir
+            .run_jj(["bookmark", "create", "-r@", bookmark])
+            .success();
+    }
+
+    let template = r#"name ++ ": " ++ if(normal_target, normal_target.author().email()) ++ "\n""#;
+    insta::assert_snapshot!(work_dir.run_jj([
+        "--config=ui.bookmark-list-sort-keys=['author-email', 'author-date-']",
+        "bookmark",
+        "list",
+        "-T",
+        template
+    ]), @r"
+    b: alice@g.c
+    a: bob@g.c
+    c: bob@g.c
+    [EOF]
+    ");
+}
+
+#[test]
+fn test_bookmark_list_sort_overriding_config() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+    for (bookmark, email) in [("c", "bob@g.c"), ("b", "alice@g.c"), ("a", "bob@g.c")] {
+        work_dir
+            .run_jj([
+                &format!("--config=user.email={email}"),
+                "new",
+                "root()",
+                "-m",
+                "fix",
+            ])
+            .success();
+        work_dir
+            .run_jj(["bookmark", "create", "-r@", bookmark])
+            .success();
+    }
+
+    let template = r#"name ++ ": " ++ if(normal_target, normal_target.author().email()) ++ "\n""#;
+    insta::assert_snapshot!(work_dir.run_jj([
+        "--config=ui.bookmark-list-sort-keys=['author-email', 'author-date-']",
+        "bookmark",
+        "list",
+        "--sort=name-", // overriding config.
+        "-T",
+        template
+    ]), @r"
+    c: bob@g.c
+    b: alice@g.c
+    a: bob@g.c
+    [EOF]
+    ");
+}
+
 #[test]
 fn test_bookmark_create_with_default_target_revision() {
     let test_env = TestEnvironment::default();
