@@ -12,27 +12,22 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::Path;
-
 use crate::common::CommandOutput;
 use crate::common::TestEnvironment;
+use crate::common::TestWorkDir;
 
 #[test]
 fn test_edit() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    std::fs::write(repo_path.join("file1"), "0").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["commit", "-m", "first"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "second"])
-        .success();
-    std::fs::write(repo_path.join("file1"), "1").unwrap();
+    let work_dir = test_env.work_dir("repo");
+    work_dir.write_file("file1", "0");
+    work_dir.run_jj(["commit", "-m", "first"]).success();
+    work_dir.run_jj(["describe", "-m", "second"]).success();
+    work_dir.write_file("file1", "1");
 
     // Errors out without argument
-    let output = test_env.run_jj_in(&repo_path, ["edit"]);
+    let output = work_dir.run_jj(["edit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the following required arguments were not provided:
@@ -46,26 +41,26 @@ fn test_edit() {
     ");
 
     // Makes the specified commit the working-copy commit
-    let output = test_env.run_jj_in(&repo_path, ["edit", "@-"]);
+    let output = work_dir.run_jj(["edit", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy now at: qpvuntsm 73383c0b first
-    Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) now at: qpvuntsm 73383c0b first
+    Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
-    let output = get_log_output(&test_env, &repo_path);
+    let output = get_log_output(&work_dir);
     insta::assert_snapshot!(output, @r"
     ○  2c910ae2d628 second
     @  73383c0b6439 first
     ◆  000000000000
     [EOF]
     ");
-    insta::assert_snapshot!(read_file(&repo_path.join("file1")), @"0");
+    insta::assert_snapshot!(work_dir.read_file("file1"), @"0");
 
     // Changes in the working copy are amended into the commit
-    std::fs::write(repo_path.join("file2"), "0").unwrap();
-    let output = get_log_output(&test_env, &repo_path);
+    work_dir.write_file("file2", "0");
+    let output = get_log_output(&work_dir);
     insta::assert_snapshot!(output, @r"
     ○  b384b2cc1883 second
     @  ff3f7b0dc386 first
@@ -81,42 +76,42 @@ fn test_edit() {
 // Windows says "Access is denied" when trying to delete the object file.
 #[cfg(unix)]
 fn test_edit_current_wc_commit_missing() {
+    use std::path::PathBuf;
+
     // Test that we get a reasonable error message when the current working-copy
     // commit is missing
+
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&repo_path, ["commit", "-m", "first"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "second"])
-        .success();
-    test_env.run_jj_in(&repo_path, ["edit", "@-"]).success();
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m", "first"]).success();
+    work_dir.run_jj(["describe", "-m", "second"]).success();
+    work_dir.run_jj(["edit", "@-"]).success();
 
-    let wc_id = test_env
-        .run_jj_in(&repo_path, ["log", "--no-graph", "-T=commit_id", "-r=@"])
+    let wc_id = work_dir
+        .run_jj(["log", "--no-graph", "-T=commit_id", "-r=@"])
         .success()
         .stdout
         .into_raw();
-    let wc_child_id = test_env
-        .run_jj_in(&repo_path, ["log", "--no-graph", "-T=commit_id", "-r=@+"])
+    let wc_child_id = work_dir
+        .run_jj(["log", "--no-graph", "-T=commit_id", "-r=@+"])
         .success()
         .stdout
         .into_raw();
     // Make the Git backend fail to read the current working copy commit
-    let commit_object_path = repo_path
-        .join(".jj")
-        .join("repo")
-        .join("store")
-        .join("git")
-        .join("objects")
-        .join(&wc_id[..2])
-        .join(&wc_id[2..]);
-    std::fs::remove_file(commit_object_path).unwrap();
+    let commit_object_path = PathBuf::from_iter([
+        ".jj",
+        "repo",
+        "store",
+        "git",
+        "objects",
+        &wc_id[..2],
+        &wc_id[2..],
+    ]);
+    work_dir.remove_file(commit_object_path);
 
     // Pass --ignore-working-copy to avoid triggering the error at snapshot time
-    let output = test_env.run_jj_in(&repo_path, ["edit", "--ignore-working-copy", &wc_child_id]);
+    let output = work_dir.run_jj(["edit", "--ignore-working-copy", &wc_child_id]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Internal error: Failed to edit a commit
@@ -129,12 +124,8 @@ fn test_edit_current_wc_commit_missing() {
     ");
 }
 
-fn read_file(path: &Path) -> String {
-    String::from_utf8(std::fs::read(path).unwrap()).unwrap()
-}
-
 #[must_use]
-fn get_log_output(test_env: &TestEnvironment, cwd: &Path) -> CommandOutput {
+fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"commit_id.short() ++ " " ++ description"#;
-    test_env.run_jj_in(cwd, ["log", "-T", template])
+    work_dir.run_jj(["log", "-T", template])
 }
