@@ -15,6 +15,7 @@
 use std::path;
 
 use indoc::formatdoc;
+use indoc::indoc;
 use test_case::test_case;
 use testutils::git;
 
@@ -1046,6 +1047,42 @@ fn test_git_clone_malformed(subprocess: bool) {
     }
 }
 
+#[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
+#[test_case(true; "spawn a git subprocess for remote calls")]
+fn test_git_clone_with_global_git_remote_config(subprocess: bool) {
+    let mut test_env = TestEnvironment::default().with_git_subprocess(subprocess);
+    test_env.work_dir("").write_file(
+        "git-config",
+        indoc! {r#"
+            [remote "origin"]
+                prune = true
+        "#},
+    );
+    test_env.add_env_var(
+        "GIT_CONFIG_GLOBAL",
+        test_env.env_root().join("git-config").to_str().unwrap(),
+    );
+
+    let root_dir = test_env.work_dir("");
+    let git_repo_path = root_dir.root().join("source");
+    let git_repo = git::init(git_repo_path);
+    set_up_non_empty_git_repo(&git_repo);
+
+    let output = root_dir.run_jj(["git", "clone", "source", "clone"]);
+    insta::allow_duplicates! {
+    insta::assert_snapshot!(output, @r#"
+    ------- stderr -------
+    Fetching into new repo in "$TEST_ENV/clone"
+    bookmark: main@origin [new] untracked
+    Setting the revset alias `trunk()` to `main@origin`
+    Working copy  (@) now at: sqpuoqvx 2ca1c979 (empty) (no description set)
+    Parent commit (@-)      : qomsplrm ebeb70d8 main | message
+    Added 1 files, modified 0 files, removed 0 files
+    [EOF]
+    "#);
+    }
+}
+
 #[test]
 fn test_git_clone_no_git_executable() {
     let test_env = TestEnvironment::default();
@@ -1094,7 +1131,6 @@ fn get_bookmark_output(work_dir: &TestWorkDir) -> CommandOutput {
 }
 
 // TODO: Remove with the `git.subprocess` setting.
-#[cfg(not(feature = "git2"))]
 #[test]
 fn test_git_clone_git2_warning() {
     let test_env = TestEnvironment::default();
@@ -1107,15 +1143,29 @@ fn test_git_clone_git2_warning() {
     set_up_non_empty_git_repo(&git_repo);
 
     let output = root_dir.run_jj(["git", "clone", "source", "clone"]);
-    insta::assert_snapshot!(output, @r#"
-    ------- stderr -------
-    Warning: Deprecated config: jj was compiled without `git.subprocess = false` support
-    Fetching into new repo in "$TEST_ENV/clone"
-    bookmark: main@origin [new] tracked
-    Setting the revset alias `trunk()` to `main@origin`
-    Working copy  (@) now at: sqpuoqvx 2ca1c979 (empty) (no description set)
-    Parent commit (@-)      : qomsplrm ebeb70d8 main | message
-    Added 1 files, modified 0 files, removed 0 files
-    [EOF]
-    "#);
+    if cfg!(feature = "git2") {
+        insta::assert_snapshot!(output, @r#"
+        ------- stderr -------
+        Warning: `git.subprocess = false` will be removed in 0.30; please report any issues you have with the default.
+        Fetching into new repo in "$TEST_ENV/clone"
+        bookmark: main@origin [new] tracked
+        Setting the revset alias `trunk()` to `main@origin`
+        Working copy  (@) now at: sqpuoqvx 2ca1c979 (empty) (no description set)
+        Parent commit (@-)      : qomsplrm ebeb70d8 main | message
+        Added 1 files, modified 0 files, removed 0 files
+        [EOF]
+        "#);
+    } else {
+        insta::assert_snapshot!(output, @r#"
+        ------- stderr -------
+        Warning: Deprecated config: jj was compiled without `git.subprocess = false` support
+        Fetching into new repo in "$TEST_ENV/clone"
+        bookmark: main@origin [new] tracked
+        Setting the revset alias `trunk()` to `main@origin`
+        Working copy  (@) now at: sqpuoqvx 2ca1c979 (empty) (no description set)
+        Parent commit (@-)      : qomsplrm ebeb70d8 main | message
+        Added 1 files, modified 0 files, removed 0 files
+        [EOF]
+        "#);
+    }
 }
