@@ -994,7 +994,7 @@ fn test_git_push_changes(subprocess: bool) {
     ");
     }
 
-    // try again with --change that moves the bookmark forward
+    // try again with --change that could move the bookmark forward
     work_dir.write_file("file", "modified5");
     work_dir
         .run_jj([
@@ -1019,9 +1019,10 @@ fn test_git_push_changes(subprocess: bool) {
     insta::allow_duplicates! {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Changes to push to origin:
-      Move sideways bookmark push-yostqsxwqrlt from c1e65d3a64ce to 38cb417ce3a6
+    Error: Bookmark already exists: push-yostqsxwqrlt
+    Hint: Use 'jj bookmark move' to move it, and 'jj git push -b push-yostqsxwqrlt [--allow-new]' to push it
     [EOF]
+    [exit status: 1]
     ");
     }
     let output = work_dir.run_jj(["status"]);
@@ -1029,8 +1030,8 @@ fn test_git_push_changes(subprocess: bool) {
     insta::assert_snapshot!(output, @r"
     Working copy changes:
     M file
-    Working copy  (@) : yostqsxw 38cb417c push-yostqsxwqrlt | bar
-    Parent commit (@-): yqosqzyt a050abf4 push-yqosqzytrlsw | foo
+    Working copy  (@) : yostqsxw 38cb417c bar
+    Parent commit (@-): yqosqzyt a050abf4 push-yostqsxwqrlt* push-yqosqzytrlsw | foo
     [EOF]
     ");
     }
@@ -1236,7 +1237,7 @@ fn test_git_push_changes_with_name_deleted_tracked(subprocess: bool) {
       @origin: kpqxywon fd39fc9d pushed
     [EOF]
     ------- stderr -------
-    Hint: Bookmarks marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj bookmark forget` to prevent this.
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
     }
@@ -2572,7 +2573,6 @@ fn get_bookmark_output(work_dir: &TestWorkDir) -> CommandOutput {
 }
 
 // TODO: Remove with the `git.subprocess` setting.
-#[cfg(not(feature = "git2"))]
 #[test]
 fn test_git_push_git2_warning() {
     let test_env = TestEnvironment::default();
@@ -2583,11 +2583,21 @@ fn test_git_push_git2_warning() {
         .run_jj(["describe", "bookmark1", "-m", "modified bookmark1 commit"])
         .success();
     let output = work_dir.run_jj(["git", "push", "--all"]);
-    insta::assert_snapshot!(output, @r#"
-    ------- stderr -------
-    Warning: Deprecated config: jj was compiled without `git.subprocess = false` support
-    Changes to push to origin:
-      Move sideways bookmark bookmark1 from d13ecdbda2a2 to 0f8dc6560f32
-    [EOF]
-    "#);
+    if cfg!(feature = "git2") {
+        insta::assert_snapshot!(output, @r"
+        ------- stderr -------
+        Warning: `git.subprocess = false` will be removed in 0.30; please report any issues you have with the default.
+        Changes to push to origin:
+          Move sideways bookmark bookmark1 from d13ecdbda2a2 to 0f8dc6560f32
+        [EOF]
+        ");
+    } else {
+        insta::assert_snapshot!(output, @r#"
+        ------- stderr -------
+        Warning: Deprecated config: jj was compiled without `git.subprocess = false` support
+        Changes to push to origin:
+          Move sideways bookmark bookmark1 from d13ecdbda2a2 to 0f8dc6560f32
+        [EOF]
+        "#);
+    }
 }
