@@ -12,26 +12,21 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::Path;
-
 use crate::common::CommandOutput;
 use crate::common::TestEnvironment;
+use crate::common::TestWorkDir;
 
 #[test]
 fn test_parallelize_no_descendants() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
     for n in 1..6 {
-        test_env
-            .run_jj_in(&workspace_path, ["commit", &format!("-m{n}")])
-            .success();
+        work_dir.run_jj(["commit", &format!("-m{n}")]).success();
     }
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=6"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["describe", "-m=6"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  02b7709cc4e9 6 parents: 5
     ○  1b2f08d76b66 5 parents: 4
     ○  e5c4cf44e237 4 parents: 3
@@ -42,10 +37,10 @@ fn test_parallelize_no_descendants() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(&workspace_path, ["parallelize", "description(1)::"])
+    work_dir
+        .run_jj(["parallelize", "description(1)::"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  4850b4629edb 6 parents:
     │ ○  87627fbb7d29 5 parents:
     ├─╯
@@ -67,17 +62,13 @@ fn test_parallelize_no_descendants() {
 fn test_parallelize_with_descendants_simple() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
     for n in 1..6 {
-        test_env
-            .run_jj_in(&workspace_path, ["commit", &format!("-m{n}")])
-            .success();
+        work_dir.run_jj(["commit", &format!("-m{n}")]).success();
     }
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=6"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["describe", "-m=6"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  02b7709cc4e9 6 parents: 5
     ○  1b2f08d76b66 5 parents: 4
     ○  e5c4cf44e237 4 parents: 3
@@ -88,13 +79,10 @@ fn test_parallelize_with_descendants_simple() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(1)::description(4)"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(1)::description(4)"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  9bc057f8b6e3 6 parents: 5
     ○        9e36a8afe793 5 parents: 1 2 3 4
     ├─┬─┬─╮
@@ -117,20 +105,16 @@ fn test_parallelize_with_descendants_simple() {
 fn test_parallelize_where_interior_has_non_target_children() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
     for n in 1..6 {
-        test_env
-            .run_jj_in(&workspace_path, ["commit", &format!("-m{n}")])
-            .success();
+        work_dir.run_jj(["commit", &format!("-m{n}")]).success();
     }
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(2)", "-m=2c"])
+    work_dir
+        .run_jj(["new", "description(2)", "-m=2c"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(5)", "-m=6"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["new", "description(5)", "-m=6"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  2508ea92308a 6 parents: 5
     ○  1b2f08d76b66 5 parents: 4
     ○  e5c4cf44e237 4 parents: 3
@@ -143,13 +127,10 @@ fn test_parallelize_where_interior_has_non_target_children() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(1)::description(4)"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(1)::description(4)"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  c9525dff9d03 6 parents: 5
     ○        b3ad09518546 5 parents: 1 2 3 4
     ├─┬─┬─╮
@@ -171,20 +152,16 @@ fn test_parallelize_where_interior_has_non_target_children() {
 fn test_parallelize_where_root_has_non_target_children() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
     for n in 1..4 {
-        test_env
-            .run_jj_in(&workspace_path, ["commit", &format!("-m{n}")])
-            .success();
+        work_dir.run_jj(["commit", &format!("-m{n}")]).success();
     }
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(1)", "-m=1c"])
+    work_dir
+        .run_jj(["new", "description(1)", "-m=1c"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(3)", "-m=4"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["new", "description(3)", "-m=4"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  9132691e6256 4 parents: 3
     ○  4cd999dfaac0 3 parents: 2
     ○  d3902619fade 2 parents: 1
@@ -194,13 +171,10 @@ fn test_parallelize_where_root_has_non_target_children() {
     ◆  000000000000 parents:
     [EOF]
     ");
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(1)::description(3)"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(1)::description(3)"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @      3397916989e7 4 parents: 1 2 3
     ├─┬─╮
     │ │ ○  1f768c1bc591 3 parents:
@@ -220,29 +194,20 @@ fn test_parallelize_where_root_has_non_target_children() {
 fn test_parallelize_with_merge_commit_child() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m", "1"])
-        .success();
+    work_dir.run_jj(["commit", "-m", "1"]).success();
     for n in 2..4 {
-        test_env
-            .run_jj_in(&workspace_path, ["commit", "-m", &n.to_string()])
-            .success();
+        work_dir.run_jj(["commit", "-m", &n.to_string()]).success();
     }
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()", "-m", "a"])
+    work_dir.run_jj(["new", "root()", "-m", "a"]).success();
+    work_dir
+        .run_jj(["new", "description(2)", "description(a)", "-m", "2a-c"])
         .success();
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["new", "description(2)", "description(a)", "-m", "2a-c"],
-        )
+    work_dir
+        .run_jj(["new", "description(3)", "-m", "4"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(3)", "-m", "4"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  99ffaf5b3984 4 parents: 3
     ○  4cd999dfaac0 3 parents: 2
     │ ○  4313cc3b476f 2a-c parents: 2 a
@@ -256,13 +221,10 @@ fn test_parallelize_with_merge_commit_child() {
     ");
 
     // After this finishes, child-2a will have three parents: "1", "2", and "a".
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(1)::description(3)"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(1)::description(3)"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @      3ee9279847a6 4 parents: 1 2 3
     ├─┬─╮
     │ │ ○  bb1bb465ccc2 3 parents:
@@ -283,17 +245,13 @@ fn test_parallelize_with_merge_commit_child() {
 fn test_parallelize_disconnected_target_commits() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
     for n in 1..3 {
-        test_env
-            .run_jj_in(&workspace_path, ["commit", &format!("-m{n}")])
-            .success();
+        work_dir.run_jj(["commit", &format!("-m{n}")]).success();
     }
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=3"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["describe", "-m=3"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  4cd999dfaac0 3 parents: 2
     ○  d3902619fade 2 parents: 1
     ○  8b64ddff700d 1 parents:
@@ -301,16 +259,13 @@ fn test_parallelize_disconnected_target_commits() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &workspace_path,
-        ["parallelize", "description(1)", "description(3)"],
-    );
+    let output = work_dir.run_jj(["parallelize", "description(1)", "description(3)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  4cd999dfaac0 3 parents: 2
     ○  d3902619fade 2 parents: 1
     ○  8b64ddff700d 1 parents:
@@ -323,32 +278,17 @@ fn test_parallelize_disconnected_target_commits() {
 fn test_parallelize_head_is_a_merge() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=0"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m=0"]).success();
+    work_dir.run_jj(["commit", "-m=1"]).success();
+    work_dir.run_jj(["commit", "-m=2"]).success();
+    work_dir.run_jj(["new", "root()"]).success();
+    work_dir.run_jj(["commit", "-m=a"]).success();
+    work_dir.run_jj(["commit", "-m=b"]).success();
+    work_dir
+        .run_jj(["new", "description(2)", "description(b)", "-m=merged-head"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=1"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=2"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=a"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=b"])
-        .success();
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["new", "description(2)", "description(b)", "-m=merged-head"],
-        )
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    1fb53c45237e merged-head parents: 2 b
     ├─╮
     │ ○  a7bf5001cfd8 b parents: a
@@ -361,10 +301,10 @@ fn test_parallelize_head_is_a_merge() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(&workspace_path, ["parallelize", "description(1)::"])
+    work_dir
+        .run_jj(["parallelize", "description(1)::"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    82131a679769 merged-head parents: 0 b
     ├─╮
     │ ○  a7bf5001cfd8 b parents: a
@@ -384,26 +324,15 @@ fn test_parallelize_head_is_a_merge() {
 fn test_parallelize_interior_target_is_a_merge() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=0"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m=0"]).success();
+    work_dir.run_jj(["describe", "-m=1"]).success();
+    work_dir.run_jj(["new", "root()", "-m=a"]).success();
+    work_dir
+        .run_jj(["new", "description(1)", "description(a)", "-m=2"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=1"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()", "-m=a"])
-        .success();
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["new", "description(1)", "description(a)", "-m=2"],
-        )
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=3"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["new", "-m=3"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  9b77792c77ac 3 parents: 2
     ○    1e29145c95fd 2 parents: 1 a
     ├─╮
@@ -415,10 +344,10 @@ fn test_parallelize_interior_target_is_a_merge() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(&workspace_path, ["parallelize", "description(1)::"])
+    work_dir
+        .run_jj(["parallelize", "description(1)::"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    042fc3f4315c 3 parents: 0 a
     ├─╮
     │ │ ○  80603361bb48 2 parents: 0 a
@@ -437,26 +366,15 @@ fn test_parallelize_interior_target_is_a_merge() {
 fn test_parallelize_root_is_a_merge() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=y"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["describe", "-m=y"]).success();
+    work_dir.run_jj(["new", "root()", "-m=x"]).success();
+    work_dir
+        .run_jj(["new", "description(y)", "description(x)", "-m=1"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()", "-m=x"])
-        .success();
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["new", "description(y)", "description(x)", "-m=1"],
-        )
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=2"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=3"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["new", "-m=2"]).success();
+    work_dir.run_jj(["new", "-m=3"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  cc239b744d01 3 parents: 2
     ○  2bf00c2ad44c 2 parents: 1
     ○    1c6853121f3c 1 parents: y x
@@ -468,13 +386,10 @@ fn test_parallelize_root_is_a_merge() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(1)::description(2)"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(1)::description(2)"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    2c7fdfa00b38 3 parents: 1 2
     ├─╮
     │ ○    3acbd32944d6 2 parents: y x
@@ -493,17 +408,11 @@ fn test_parallelize_root_is_a_merge() {
 fn test_parallelize_multiple_heads() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=0"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=1"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(0)", "-m=2"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m=0"]).success();
+    work_dir.run_jj(["describe", "-m=1"]).success();
+    work_dir.run_jj(["new", "description(0)", "-m=2"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  97d7522f40e8 2 parents: 0
     │ ○  0c058af014a6 1 parents: 0
     ├─╯
@@ -512,10 +421,10 @@ fn test_parallelize_multiple_heads() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(&workspace_path, ["parallelize", "description(0)::"])
+    work_dir
+        .run_jj(["parallelize", "description(0)::"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  e84481c26195 2 parents:
     │ ○  6270540ee067 1 parents:
     ├─╯
@@ -533,17 +442,11 @@ fn test_parallelize_multiple_heads() {
 fn test_parallelize_multiple_heads_with_and_without_children() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=0"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=1"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "description(0)", "-m=2"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m=0"]).success();
+    work_dir.run_jj(["describe", "-m=1"]).success();
+    work_dir.run_jj(["new", "description(0)", "-m=2"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  97d7522f40e8 2 parents: 0
     │ ○  0c058af014a6 1 parents: 0
     ├─╯
@@ -552,13 +455,10 @@ fn test_parallelize_multiple_heads_with_and_without_children() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(0)", "description(1)"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(0)", "description(1)"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  97d7522f40e8 2 parents: 0
     ○  745bea8029c1 0 parents:
     │ ○  6270540ee067 1 parents:
@@ -572,23 +472,14 @@ fn test_parallelize_multiple_heads_with_and_without_children() {
 fn test_parallelize_multiple_roots() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["describe", "-m=1"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["describe", "-m=1"]).success();
+    work_dir.run_jj(["new", "root()", "-m=a"]).success();
+    work_dir
+        .run_jj(["new", "description(1)", "description(a)", "-m=2"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()", "-m=a"])
-        .success();
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["new", "description(1)", "description(a)", "-m=2"],
-        )
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=3"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["new", "-m=3"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  34da938ad94a 3 parents: 2
     ○    85d5043b881d 2 parents: 1 a
     ├─╮
@@ -600,10 +491,8 @@ fn test_parallelize_multiple_roots() {
     ");
 
     // Succeeds because the roots have the same parents.
-    test_env
-        .run_jj_in(&workspace_path, ["parallelize", "root().."])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    work_dir.run_jj(["parallelize", "root().."]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  3c90598481cd 3 parents:
     │ ○  b96aa55582e5 2 parents:
     ├─╯
@@ -620,29 +509,15 @@ fn test_parallelize_multiple_roots() {
 fn test_parallelize_multiple_heads_with_different_children() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=1"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=2"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=3"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=a"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=b"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=c"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m=1"]).success();
+    work_dir.run_jj(["commit", "-m=2"]).success();
+    work_dir.run_jj(["commit", "-m=3"]).success();
+    work_dir.run_jj(["new", "root()"]).success();
+    work_dir.run_jj(["commit", "-m=a"]).success();
+    work_dir.run_jj(["commit", "-m=b"]).success();
+    work_dir.run_jj(["commit", "-m=c"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  4bc4dace0e65 parents: c
     ○  63b0da9212c0 c parents: b
     ○  a7bf5001cfd8 b parents: a
@@ -655,17 +530,14 @@ fn test_parallelize_multiple_heads_with_different_children() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            [
-                "parallelize",
-                "description(1)::description(2)",
-                "description(a)::description(b)",
-            ],
-        )
+    work_dir
+        .run_jj([
+            "parallelize",
+            "description(1)::description(2)",
+            "description(a)::description(b)",
+        ])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  f6c9d9ee3db8 parents: c
     ○    62661d5f0c77 c parents: a b
     ├─╮
@@ -687,29 +559,16 @@ fn test_parallelize_multiple_heads_with_different_children() {
 fn test_parallelize_multiple_roots_with_different_parents() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=1"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["commit", "-m=1"]).success();
+    work_dir.run_jj(["commit", "-m=2"]).success();
+    work_dir.run_jj(["new", "root()"]).success();
+    work_dir.run_jj(["commit", "-m=a"]).success();
+    work_dir.run_jj(["commit", "-m=b"]).success();
+    work_dir
+        .run_jj(["new", "description(2)", "description(b)", "-m=merged-head"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=2"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "root()"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=a"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["commit", "-m=b"])
-        .success();
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["new", "description(2)", "description(b)", "-m=merged-head"],
-        )
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    ba4297d53c1a merged-head parents: 2 b
     ├─╮
     │ ○  6577defaca2d b parents: a
@@ -721,13 +580,10 @@ fn test_parallelize_multiple_roots_with_different_parents() {
     [EOF]
     ");
 
-    test_env
-        .run_jj_in(
-            &workspace_path,
-            ["parallelize", "description(2)::", "description(b)::"],
-        )
+    work_dir
+        .run_jj(["parallelize", "description(2)::", "description(b)::"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    0943ed52b3ed merged-head parents: 1 a
     ├─╮
     │ │ ○  6577defaca2d b parents: a
@@ -746,32 +602,22 @@ fn test_parallelize_multiple_roots_with_different_parents() {
 fn test_parallelize_complex_nonlinear_target() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=0", "root()"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["new", "-m=0", "root()"]).success();
+    work_dir.run_jj(["new", "-m=1", "description(0)"]).success();
+    work_dir.run_jj(["new", "-m=2", "description(0)"]).success();
+    work_dir.run_jj(["new", "-m=3", "description(0)"]).success();
+    work_dir.run_jj(["new", "-m=4", "all:heads(..)"]).success();
+    work_dir
+        .run_jj(["new", "-m=1c", "description(1)"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=1", "description(0)"])
+    work_dir
+        .run_jj(["new", "-m=2c", "description(2)"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=2", "description(0)"])
+    work_dir
+        .run_jj(["new", "-m=3c", "description(3)"])
         .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=3", "description(0)"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=4", "all:heads(..)"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=1c", "description(1)"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=2c", "description(2)"])
-        .success();
-    test_env
-        .run_jj_in(&workspace_path, ["new", "-m=3c", "description(3)"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  b043eb81416c 3c parents: 3
     │ ○    48277ee9afe0 4 parents: 3 2 1
     ╭─┼─╮
@@ -789,18 +635,15 @@ fn test_parallelize_complex_nonlinear_target() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &workspace_path,
-        ["parallelize", "description(0)::description(4)"],
-    );
+    let output = work_dir.run_jj(["parallelize", "description(0)::description(4)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy now at: yostqsxw 59a216e5 (empty) 3c
-    Parent commit      : rlvkpnrz 745bea80 (empty) 0
-    Parent commit      : mzvwutvl cb944786 (empty) 3
+    Working copy  (@) now at: yostqsxw 59a216e5 (empty) 3c
+    Parent commit (@-)      : rlvkpnrz 745bea80 (empty) 0
+    Parent commit (@-)      : mzvwutvl cb944786 (empty) 3
     [EOF]
     ");
-    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    59a216e537c4 3c parents: 0 3
     ├─╮
     │ ○  cb9447869bf0 3 parents:
@@ -822,7 +665,7 @@ fn test_parallelize_complex_nonlinear_target() {
 }
 
 #[must_use]
-fn get_log_output(test_env: &TestEnvironment, cwd: &Path) -> CommandOutput {
+fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"
     separate(" ",
         commit_id.short(),
@@ -830,5 +673,5 @@ fn get_log_output(test_env: &TestEnvironment, cwd: &Path) -> CommandOutput {
         "parents:",
         parents.map(|c|c.description().first_line())
     )"#;
-    test_env.run_jj_in(cwd, ["log", "-T", template])
+    work_dir.run_jj(["log", "-T", template])
 }
