@@ -46,7 +46,6 @@ fn test_restore() {
     let output = work_dir.run_jj(["restore"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created kkmpptxz 370d81ea (empty) (no description set)
     Working copy  (@) now at: kkmpptxz 370d81ea (empty) (no description set)
     Parent commit (@-)      : rlvkpnrz ef160660 (no description set)
     Added 1 files, modified 1 files, removed 1 files
@@ -65,7 +64,6 @@ fn test_restore() {
     let output = work_dir.run_jj(["restore", "-c=@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created rlvkpnrz b9b6011e (empty) (no description set)
     Rebased 1 descendant commits
     Working copy  (@) now at: kkmpptxz 5b361547 (conflict) (no description set)
     Parent commit (@-)      : rlvkpnrz b9b6011e (empty) (no description set)
@@ -89,7 +87,6 @@ fn test_restore() {
     let output = work_dir.run_jj(["restore", "--from", "@--"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created kkmpptxz 1154634b (no description set)
     Working copy  (@) now at: kkmpptxz 1154634b (no description set)
     Parent commit (@-)      : rlvkpnrz ef160660 (no description set)
     Added 1 files, modified 0 files, removed 2 files
@@ -106,7 +103,6 @@ fn test_restore() {
     let output = work_dir.run_jj(["restore", "--into", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created rlvkpnrz ad805965 (no description set)
     Rebased 1 descendant commits
     Working copy  (@) now at: kkmpptxz 3fcdcbf2 (empty) (no description set)
     Parent commit (@-)      : rlvkpnrz ad805965 (no description set)
@@ -127,7 +123,6 @@ fn test_restore() {
     let output = work_dir.run_jj(["restore", "--from", "@", "--into", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created rlvkpnrz f256040a (no description set)
     Rebased 1 descendant commits
     Working copy  (@) now at: kkmpptxz 9c6f2083 (empty) (no description set)
     Parent commit (@-)      : rlvkpnrz f256040a (no description set)
@@ -148,7 +143,6 @@ fn test_restore() {
     let output = work_dir.run_jj(["restore", "file2", "file3"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created kkmpptxz 4ad35a2f (no description set)
     Working copy  (@) now at: kkmpptxz 4ad35a2f (no description set)
     Parent commit (@-)      : rlvkpnrz ef160660 (no description set)
     Added 0 files, modified 1 files, removed 1 files
@@ -212,7 +206,6 @@ fn test_restore_conflicted_merge() {
     let output = work_dir.run_jj(["restore", "file"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created vruxwmqv 25a37060 conflict | (conflict) (empty) conflict
     Working copy  (@) now at: vruxwmqv 25a37060 conflict | (conflict) (empty) conflict
     Parent commit (@-)      : zsuskuln aa493daf a | a
     Parent commit (@-)      : royxmykx db6a4daf b | b
@@ -252,7 +245,6 @@ fn test_restore_conflicted_merge() {
     let output = work_dir.run_jj(["restore"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created vruxwmqv f2c82b9c conflict | (conflict) (empty) conflict
     Working copy  (@) now at: vruxwmqv f2c82b9c conflict | (conflict) (empty) conflict
     Parent commit (@-)      : zsuskuln aa493daf a | a
     Parent commit (@-)      : royxmykx db6a4daf b | b
@@ -305,7 +297,6 @@ fn test_restore_restore_descendants() {
     let output = work_dir.run_jj(["restore", "-c", "b", "file", "--restore-descendants"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created royxmykx 3fd5aa05 b | b
     Rebased 1 descendant commits (while preserving their content)
     Working copy  (@) now at: vruxwmqv bf5491a0 ab | ab
     Parent commit (@-)      : zsuskuln aa493daf a | a
@@ -386,7 +377,6 @@ fn test_restore_interactive() {
     let output = work_dir.run_jj(["restore", "-i", "--from=@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created zsuskuln bccde490 b | b
     Working copy  (@) now at: zsuskuln bccde490 b | b
     Parent commit (@-)      : rlvkpnrz 186caaef a | a
     Added 0 files, modified 1 files, removed 1 files
@@ -420,7 +410,6 @@ fn test_restore_interactive() {
     let output = work_dir.run_jj(["restore", "--tool=fake-diff-editor"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created zsuskuln 5921de19 b | b
     Working copy  (@) now at: zsuskuln 5921de19 b | b
     Parent commit (@-)      : rlvkpnrz 186caaef a | a
     Added 0 files, modified 1 files, removed 1 files
@@ -486,7 +475,6 @@ fn test_restore_interactive_merge() {
     let output = work_dir.run_jj(["restore", "-i"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created royxmykx 72e0cbf4 c | c
     Working copy  (@) now at: royxmykx 72e0cbf4 c | c
     Parent commit (@-)      : rlvkpnrz 79c1b823 a | a
     Parent commit (@-)      : zsuskuln 29e70804 b | b
@@ -561,7 +549,6 @@ fn test_restore_interactive_with_paths() {
     let output = work_dir.run_jj(["restore", "-i", "file1", "file2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Created zsuskuln 7187da33 b | b
     Working copy  (@) now at: zsuskuln 7187da33 b | b
     Parent commit (@-)      : rlvkpnrz 186caaef a | a
     Added 0 files, modified 1 files, removed 0 files
