@@ -23,7 +23,7 @@ pub trait ContentHash {
 
 /// The 512-bit BLAKE2b content hash
 pub fn blake2b_hash(x: &(impl ContentHash + ?Sized)) -> digest::Output<Blake2b512> {
-    use digest::Digest;
+    use digest::Digest as _;
     let mut hasher = Blake2b512::default();
     x.hash(&mut hasher);
     hasher.finalize()
@@ -85,12 +85,18 @@ impl<T: ContentHash> ContentHash for Vec<T> {
     }
 }
 
-impl ContentHash for String {
+impl ContentHash for str {
     fn hash(&self, state: &mut impl DigestUpdate) {
         self.as_bytes().hash(state);
     }
 }
 
+impl ContentHash for String {
+    fn hash(&self, state: &mut impl DigestUpdate) {
+        self.as_str().hash(state);
+    }
+}
+
 impl<T: ContentHash> ContentHash for Option<T> {
     fn hash(&self, state: &mut impl DigestUpdate) {
         match self {
