@@ -18,12 +18,12 @@ use std::any::Any;
 use std::collections::HashSet;
 use std::fs;
 use std::io;
-use std::io::Write;
+use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
 
-use itertools::Itertools;
+use itertools::Itertools as _;
 use tempfile::NamedTempFile;
 use thiserror::Error;
 
@@ -40,13 +40,13 @@ use crate::file_util;
 use crate::file_util::persist_content_addressed_temp_file;
 use crate::file_util::IoResultExt as _;
 use crate::file_util::PathError;
-use crate::index::Index;
+use crate::index::Index as _;
 use crate::index::IndexReadError;
 use crate::index::IndexStore;
 use crate::index::IndexWriteError;
 use crate::index::MutableIndex;
 use crate::index::ReadonlyIndex;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 use crate::op_store::OpStoreError;
 use crate::op_store::OperationId;
 use crate::operation::Operation;
