@@ -32,7 +32,6 @@ use crate::backend::MergedTreeId;
 use crate::commit::Commit;
 use crate::commit::CommitIteratorExt as _;
 use crate::commit_builder::CommitBuilder;
-use crate::dag_walk;
 use crate::index::Index;
 use crate::index::IndexError;
 use crate::matchers::Matcher;
@@ -403,6 +402,7 @@ pub struct MoveCommitsStats {
     pub num_abandoned: u32,
 }
 
+#[derive(Clone, Debug)]
 pub enum MoveCommitsTarget {
     /// The commits to be moved. Commits should be mutable and in reverse
     /// topological order.
@@ -445,12 +445,11 @@ pub fn move_commits(
                 RevsetExpression::commits(target_commits.iter().ids().cloned().collect_vec())
                     .connected()
                     .evaluate(mut_repo)
-                    .map_err(|err| err.expect_backend_error())?
+                    .map_err(|err| err.into_backend_error())?
                     .iter()
                     .commits(mut_repo.store())
                     .try_collect()
-                    // TODO: Return evaluation error to caller
-                    .map_err(|err| err.expect_backend_error())?;
+                    .map_err(|err| err.into_backend_error())?;
             connected_target_commits_internal_parents =
                 compute_internal_parents_within(&target_commit_ids, &connected_target_commits);
 
@@ -470,12 +469,11 @@ pub fn move_commits(
             target_commits = RevsetExpression::commits(roots.iter().ids().cloned().collect_vec())
                 .descendants()
                 .evaluate(mut_repo)
-                .map_err(|err| err.expect_backend_error())?
+                .map_err(|err| err.into_backend_error())?
                 .iter()
                 .commits(mut_repo.store())
                 .try_collect()
-                // TODO: Return evaluation error to caller
-                .map_err(|err| err.expect_backend_error())?;
+                .map_err(|err| err.into_backend_error())?;
             target_commit_ids = target_commits.iter().ids().cloned().collect();
 
             connected_target_commits = target_commits.iter().cloned().collect_vec();
@@ -530,12 +528,11 @@ pub fn move_commits(
                         .children(),
                 )
                 .evaluate(mut_repo)
-                .map_err(|err| err.expect_backend_error())?
+                .map_err(|err| err.into_backend_error())?
                 .iter()
                 .commits(mut_repo.store())
                 .try_collect()
-                // TODO: Return evaluation error to caller
-                .map_err(|err| err.expect_backend_error())?;
+                .map_err(|err| err.into_backend_error())?;
 
         // For all commits in the target set, compute its transitive descendant commits
         // which are outside of the target set by up to 1 generation.
@@ -638,24 +635,14 @@ pub fn move_commits(
     // new children commits (if any), and their descendants.
     let mut roots = target_roots.iter().cloned().collect_vec();
     roots.extend(new_children.iter().ids().cloned());
-    let to_visit_expression = RevsetExpression::commits(roots).descendants();
-    let to_visit: Vec<_> = to_visit_expression
-        .evaluate(mut_repo)
-        .map_err(|err| err.expect_backend_error())?
-        .iter()
-        .commits(mut_repo.store())
-        .try_collect()
-        // TODO: Return evaluation error to caller
-        .map_err(|err| err.expect_backend_error())?;
-    let to_visit_commits: IndexMap<_, _> = to_visit
-        .into_iter()
-        .map(|commit| (commit.id().clone(), commit))
-        .collect();
 
-    let to_visit_commits_new_parents: HashMap<_, _> = to_visit_commits
+    let descendants = mut_repo.find_descendants_for_rebase(roots.clone())?;
+    let commit_new_parents_map: HashMap<_, _> = descendants
         .iter()
-        .map(|(commit_id, commit)| {
-            let new_parents =
+        .map(|commit| {
+            let commit_id = commit.id();
+            let new_parent_ids =
+
             // New child of the rebased target commits.
             if let Some(new_child_parents) = new_children_parents.get(commit_id) {
                 new_child_parents.clone()
@@ -708,32 +695,10 @@ pub fn move_commits(
             } else {
                 commit.parent_ids().iter().cloned().collect_vec()
             };
-
-            (commit_id.clone(), new_parents)
+            (commit.id().clone(), new_parent_ids)
         })
         .collect();
 
-    // Re-compute the order of commits to visit, such that each commit's new parents
-    // must be visited first.
-    let mut visited: HashSet<CommitId> = HashSet::new();
-    let mut to_visit = dag_walk::topo_order_reverse(
-        to_visit_commits.keys().cloned().collect_vec(),
-        |commit_id| commit_id.clone(),
-        |commit_id| -> Vec<CommitId> {
-            visited.insert(commit_id.clone());
-            to_visit_commits_new_parents
-                .get(commit_id)
-                .cloned()
-                .unwrap()
-                .iter()
-                // Only add parents which are in the set to be visited and have not already been
-                // visited.
-                .filter(|&id| to_visit_commits.contains_key(id) && !visited.contains(id))
-                .cloned()
-                .collect()
-        },
-    );
-
     let mut num_rebased_targets = 0;
     let mut num_rebased_descendants = 0;
     let mut num_skipped_rebases = 0;
@@ -746,35 +711,36 @@ pub fn move_commits(
         simplify_ancestor_merge: options.simplify_ancestor_merge,
     };
 
-    // Rebase each commit onto its new parents in the reverse topological order
-    // computed above.
-    while let Some(old_commit_id) = to_visit.pop() {
-        let old_commit = to_visit_commits.get(&old_commit_id).unwrap();
-        let parent_ids = to_visit_commits_new_parents.get(&old_commit_id).unwrap();
-        let new_parent_ids = mut_repo.new_parents(parent_ids);
-        let rewriter = CommitRewriter::new(mut_repo, old_commit.clone(), new_parent_ids);
-        if rewriter.parents_changed() {
-            let is_target_commit = target_commit_ids.contains(&old_commit_id);
-            let rebased_commit = rebase_commit_with_options(
-                rewriter,
-                if is_target_commit {
-                    options
+    mut_repo.transform_commits(
+        descendants,
+        &commit_new_parents_map,
+        &options.rewrite_refs,
+        |rewriter| {
+            let old_commit_id = rewriter.old_commit().id();
+            if rewriter.parents_changed() {
+                let is_target_commit = target_commit_ids.contains(old_commit_id);
+                let rebased_commit = rebase_commit_with_options(
+                    rewriter,
+                    if is_target_commit {
+                        options
+                    } else {
+                        rebase_descendant_options
+                    },
+                )?;
+                if let RebasedCommit::Abandoned { .. } = rebased_commit {
+                    num_abandoned += 1;
+                } else if is_target_commit {
+                    num_rebased_targets += 1;
                 } else {
-                    rebase_descendant_options
-                },
-            )?;
-            if let RebasedCommit::Abandoned { .. } = rebased_commit {
-                num_abandoned += 1;
-            } else if is_target_commit {
-                num_rebased_targets += 1;
+                    num_rebased_descendants += 1;
+                }
             } else {
-                num_rebased_descendants += 1;
+                num_skipped_rebases += 1;
             }
-        } else {
-            num_skipped_rebases += 1;
-        }
-    }
-    mut_repo.update_rewritten_references(&options.rewrite_refs)?;
+
+            Ok(())
+        },
+    )?;
 
     Ok(MoveCommitsStats {
         num_rebased_targets,
@@ -801,6 +767,10 @@ pub struct DuplicateCommitsStats {
 /// `children_commit_ids` is not empty, the `children_commit_ids` will be
 /// rebased onto the heads of the duplicated target commits.
 ///
+/// If `target_descriptions` is not empty, it will be consulted to retrieve the
+/// new descriptions of the target commits, falling back to the original if
+/// the map does not contain an entry for a given commit.
+///
 /// This assumes that commits in `children_commit_ids` can be rewritten. There
 /// should also be no cycles in the resulting graph, i.e. `children_commit_ids`
 /// should not be ancestors of `parent_commit_ids`. Commits in `target_commits`
@@ -808,6 +778,7 @@ pub struct DuplicateCommitsStats {
 pub fn duplicate_commits(
     mut_repo: &mut MutableRepo,
     target_commits: &[CommitId],
+    target_descriptions: &HashMap<CommitId, String>,
     parent_commit_ids: &[CommitId],
     children_commit_ids: &[CommitId],
 ) -> BackendResult<DuplicateCommitsStats> {
@@ -824,12 +795,11 @@ pub fn duplicate_commits(
         RevsetExpression::commits(target_commit_ids.iter().cloned().collect_vec())
             .connected()
             .evaluate(mut_repo)
-            .map_err(|err| err.expect_backend_error())?
+            .map_err(|err| err.into_backend_error())?
             .iter()
             .commits(mut_repo.store())
             .try_collect()
-            // TODO: Return evaluation error to caller
-            .map_err(|err| err.expect_backend_error())?;
+            .map_err(|err| err.into_backend_error())?;
 
     // Commits in the target set should only have other commits in the set as
     // parents, except the roots of the set, which persist their original
@@ -879,11 +849,13 @@ pub fn duplicate_commits(
                 })
                 .collect()
         };
-        let new_commit = CommitRewriter::new(mut_repo, original_commit, new_parent_ids)
+        let mut new_commit_builder = CommitRewriter::new(mut_repo, original_commit, new_parent_ids)
             .rebase()?
-            .generate_new_change_id()
-            .write()?;
-        duplicated_old_to_new.insert(original_commit_id.clone(), new_commit);
+            .generate_new_change_id();
+        if let Some(desc) = target_descriptions.get(original_commit_id) {
+            new_commit_builder = new_commit_builder.set_description(desc);
+        }
+        duplicated_old_to_new.insert(original_commit_id.clone(), new_commit_builder.write()?);
     }
 
     // Replace the original commit IDs in `target_head_ids` with the duplicated
@@ -934,9 +906,14 @@ pub fn duplicate_commits(
 ///
 /// Commits in `target_commits` should be in reverse topological order (children
 /// before parents).
+///
+/// If `target_descriptions` is not empty, it will be consulted to retrieve the
+/// new descriptions of the target commits, falling back to the original if
+/// the map does not contain an entry for a given commit.
 pub fn duplicate_commits_onto_parents(
     mut_repo: &mut MutableRepo,
     target_commits: &[CommitId],
+    target_descriptions: &HashMap<CommitId, String>,
 ) -> BackendResult<DuplicateCommitsStats> {
     if target_commits.is_empty() {
         return Ok(DuplicateCommitsStats::default());
@@ -958,12 +935,14 @@ pub fn duplicate_commits_onto_parents(
                     .clone()
             })
             .collect();
-        let new_commit = mut_repo
+        let mut new_commit_builder = mut_repo
             .rewrite_commit(&original_commit)
             .generate_new_change_id()
-            .set_parents(new_parent_ids)
-            .write()?;
-        duplicated_old_to_new.insert(original_commit_id.clone(), new_commit);
+            .set_parents(new_parent_ids);
+        if let Some(desc) = target_descriptions.get(original_commit_id) {
+            new_commit_builder = new_commit_builder.set_description(desc);
+        }
+        duplicated_old_to_new.insert(original_commit_id.clone(), new_commit_builder.write()?);
     }
 
     Ok(DuplicateCommitsStats {
