@@ -16,7 +16,7 @@
 
 use std::ffi::OsString;
 use std::fmt::Debug;
-use std::io::Write;
+use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
 use std::process::Command;
@@ -130,6 +130,13 @@ impl SshBackend {
 
     fn create_command(&self) -> Command {
         let mut command = Command::new(&self.program);
+        // Hide console window on Windows (https://stackoverflow.com/a/60958956)
+        #[cfg(windows)]
+        {
+            use std::os::windows::process::CommandExt;
+            const CREATE_NO_WINDOW: u32 = 0x08000000;
+            command.creation_flags(CREATE_NO_WINDOW);
+        }
 
         command
             .stdin(Stdio::piped())
@@ -278,7 +285,7 @@ impl SigningBackend for SshBackend {
 #[cfg(test)]
 mod tests {
     use std::fs::File;
-    use std::io::Read;
+    use std::io::Read as _;
 
     use super::*;
 
